package com.s3ai.corporate_app2.controllers

import com.s3ai.corporate_app2.Cinema
import com.s3ai.corporate_app2.CinemaService
import com.s3ai.corporate_app2.jms.JMSPublisher
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Controller
import org.springframework.ui.Model
import org.springframework.web.bind.annotation.*
import org.springframework.web.servlet.view.RedirectView
import java.util.UUID.fromString

@Controller
@RequestMapping("/cinemas")
class CinemasController {
    @Autowired
    lateinit var cinemaService: CinemaService

    @Autowired
    lateinit var publisher: JMSPublisher

    @GetMapping("/list")
    fun getCinemasBrowsePage(model: Model): String? {
        val items = getAllItems(cinemaService, publisher)
        model.addAttribute("cinemas", items)
        return "cinemas/list"
    }

    @GetMapping("/edit")
    fun getCinemaEditPage(@RequestParam(name = "id", required = false, defaultValue = "") id: String, model: Model): String? {
        val cinema: Cinema?
        if (id.isEmpty()) {
            cinema = Cinema()
            model.addAttribute("action", "Create")
        } else {
            cinema = cinemaService.findById(fromString(id))
            model.addAttribute("action", "Edit")
        }
        model.addAttribute("cinema", cinema)
        return "cinemas/edit"
    }

    @PostMapping("/update")
    fun updateCinema(@ModelAttribute cinema: Cinema): RedirectView {
        updateItem(cinemaService, cinema, cinema.id, publisher)
        return RedirectView("/cinemas/list")
    }
}