package com.s3ai.corporate_app2.controllers

import com.fasterxml.jackson.dataformat.xml.XmlMapper
import com.s3ai.corporate_app2.CinemaServices
import com.s3ai.corporate_app2.jms.JMSPublisher
import com.s3ai.corporate_app2.jms.persistance.JMSAction
import com.s3ai.corporate_app2.jms.persistance.JMSEntityAction
import org.springframework.util.ResourceUtils
import java.io.StringReader
import java.io.StringWriter
import java.util.*
import javax.servlet.http.HttpServletResponse
import javax.xml.transform.TransformerFactory
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.stream.StreamSource

fun publishMessage(publisher: JMSPublisher, entity: String, actionJMS: JMSEntityAction, description: String) {
    val jmsAction = JMSAction()
    jmsAction.entity = entity
    jmsAction.actionJMS = actionJMS
    jmsAction.actionDescription = description
    publisher.publish(jmsAction)
}

fun <T> deleteInstance(id: String, jpaService: CinemaServices<T>, response: HttpServletResponse, publisher: JMSPublisher): String? {
    var responseString: String? = null
    try {
        val itemId = UUID.fromString(id)
        val item = jpaService.findById(itemId)
        if (null != item) {
            jpaService.delete(item)
            publishMessage(publisher, jpaService.itemName(), JMSEntityAction.DELETE, "Deleted object: $item")
        } else {
            response.status = HttpServletResponse.SC_NOT_FOUND
            responseString = "${jpaService.itemName()} was not Found"
        }
    } catch (e: IllegalArgumentException) {
        response.status = HttpServletResponse.SC_BAD_REQUEST
        responseString = e.localizedMessage
    }
    return responseString
}

fun <T : Any> getInstance(id: UUID, jpaService: CinemaServices<T>, response: HttpServletResponse, publisher: JMSPublisher): T? {
    val item = jpaService.findById(id)
    publishMessage(publisher, jpaService.itemName(), JMSEntityAction.QUERY, "Queried object with id: $id, Found: $item")
    if (null != item) {
        return item
    } else response.status = HttpServletResponse.SC_NOT_FOUND
    return null
}

fun <T : Any> getAllItems(jpaService: CinemaServices<T>, publisher: JMSPublisher): MutableList<T> {
    val items = jpaService.findAll()
    publishMessage(publisher, jpaService.itemName(), JMSEntityAction.QUERY, "Queried all items, found ${items.size}.")
    return items
}

fun <T : Any> updateItem(jpaService: CinemaServices<T>, item: T, item_id: UUID?, publisher: JMSPublisher) {
    var action = JMSEntityAction.UPDATE
    val updatedItem = jpaService.save(item)
    val description = "New item data: $updatedItem"
    if (null == item_id) action = JMSEntityAction.CREATE
    publishMessage(publisher, jpaService.itemName(), action, description)
}

fun xsltTransform(item: Any, resource_name: String): String {
    val xmlMapper = XmlMapper()
    val xslt = StreamSource(ResourceUtils.getFile("classpath:xslt/$resource_name"))
    val xml = StreamSource(StringReader(xmlMapper.writeValueAsString(item)))
    val transformer = TransformerFactory.newInstance().newTransformer(xslt)
    val sw = StringWriter()
    transformer.transform(xml, StreamResult(sw))
    return sw.toString()
}