package com.s3ai.corporate_app2.jms

import com.fasterxml.jackson.module.kotlin.jacksonObjectMapper
import com.fasterxml.jackson.module.kotlin.readValue
import com.s3ai.corporate_app2.jms.persistance.JMSAction
import com.s3ai.corporate_app2.jms.persistance.JMSActionRepository
import com.s3ai.corporate_app2.jms.persistance.SubscriptionTypeRepository
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.annotation.Value
import org.springframework.jms.annotation.JmsListener
import org.springframework.mail.javamail.JavaMailSender
import org.springframework.mail.javamail.MimeMessageHelper
import org.springframework.messaging.MessagingException
import org.springframework.stereotype.Component


@Component
class JmsConsumer {

    @Value("\${spring.mail.username}")
    private lateinit var sourceUser: String

    @Autowired
    private lateinit var jmsActionService: JMSActionRepository

    @Autowired
    private lateinit var sender: JavaMailSender

    @Autowired
    private lateinit var subTypeRepo: SubscriptionTypeRepository

    private final val mapper = jacksonObjectMapper()

    fun sendEmail(action: JMSAction, targetUsers: Array<String>): Boolean {
        val message = sender.createMimeMessage()
        val helper = MimeMessageHelper(message)
        try {
            helper.setFrom(sourceUser)
            helper.setTo(targetUsers)
            helper.setText("Entity ${action.entity} was ${action.actionJMS.str}. ${action.actionDescription}")
            helper.setSubject("Cinemas service notification")
            sender.send(message)
        } catch (e: MessagingException) {
            e.printStackTrace()
            return false
        }
        return true

    }

    fun notifySubscribers(action: JMSAction) {
        val users = subTypeRepo.findSubscriptionTypeByType(action.actionJMS)
        val emails = users.map { u -> u.subscriber.email!! }.distinct().toTypedArray()
        if (emails.isNotEmpty()) sendEmail(action, emails)
    }

    @JmsListener(destination = "cinemas.queue")
    fun consume(action_text: String) {
        val action: JMSAction = mapper.readValue(action_text)
        jmsActionService.save(action)
        notifySubscribers(action)
    }
}