package com.s3ai.corporate_app2.jms.persistance

import com.fasterxml.jackson.annotation.JsonAutoDetect
import org.hibernate.annotations.OnDelete
import org.hibernate.annotations.OnDeleteAction
import java.util.*
import javax.persistence.*

enum class JMSEntityAction(val str: String) {
    UPDATE("updated"),
    QUERY("queried"),
    CREATE("created"),
    DELETE("deleted")
}

@Entity
@Table(name = "jms_action")
@JsonAutoDetect(fieldVisibility = JsonAutoDetect.Visibility.ANY)
class JMSAction {
    @Id
    @Column(name = "ID")
    @GeneratedValue(strategy = GenerationType.AUTO)
    var id: UUID? = null
    @Enumerated(EnumType.STRING)
    @Column(length = 8, name = "action")
    lateinit var actionJMS: JMSEntityAction
    @Column(name = "entity")
    lateinit var entity: String
    @Column(name = "description", columnDefinition = "TEXT")
    lateinit var actionDescription: String
    @Column(name = "delivered")
    var delivered: Boolean = true

    override fun toString(): String {
        return "JMSAction{id=$id, action=$actionJMS, entity='$entity', actionDescription='$actionDescription'}"
    }
}

@Entity
@Table(name = "subscriber")
class Subscriber {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    var id: UUID? = null

    @Column(name = "email")
    var email: String? = null

    @OneToMany
    var types: List<SubscriptionType> = mutableListOf()
}

@Entity
@Table(name = "subscription_type")
class SubscriptionType {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    var id: Long? = null

    @Enumerated(EnumType.STRING)
    @Column(length = 8, name = "type")
    lateinit var type: JMSEntityAction

    @OneToOne
    @OnDelete(action = OnDeleteAction.CASCADE)
    lateinit var subscriber: Subscriber
}