package com.s3ai.corporate_app2

import com.s3ai.corporate_app2.controllers.CinemasController
import com.s3ai.corporate_app2.controllers.rest.CinemaApiController
import org.assertj.core.api.Assertions.assertThat
import org.junit.jupiter.api.Test
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.jdbc.Sql
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.status
import org.springframework.ui.ExtendedModelMap
import java.util.*


@SpringBootTest
@AutoConfigureMockMvc
class CinemasApiTest {
    @Autowired
    private lateinit var controller: CinemasController

    @Autowired
    private lateinit var mvc: MockMvc

    @Autowired
    private lateinit var apiController: CinemaApiController

    @Test
    @Throws(Exception::class)
    fun contextLoads() {
        assertThat(controller).isNotNull
        assertThat(apiController).isNotNull
    }

    @Test
    @Throws(Exception::class)
    fun shouldUpdateCinema() {
        val cinema = Cinema()
        val id = UUID.randomUUID()
        cinema.id = id
        cinema.name = "test"
        controller.updateCinema(cinema)
        val cinemaFromBase = controller.cinemaService.findById(id)
        assertThat(cinemaFromBase?.equals(cinema))
    }

    @Test
    @Throws(Exception::class)
    fun shouldRedirectToCreateCinemaIfIdNotExists() {
        val model = ExtendedModelMap()
        val redirect = controller.getCinemaEditPage(UUID.randomUUID().toString(), model)
        assertThat(redirect.equals("cinemas/edit"))
        assertThat(model.getValue("action") == "Create")
    }

    @Test
    @Throws(Exception::class)
    fun shouldRedirectToEditCinemaIfIdExists() {
        val cinema = Cinema()
        val id = UUID.randomUUID()
        cinema.id = id
        cinema.name = "test"
        controller.updateCinema(cinema)
        val model = ExtendedModelMap()
        val redirect = controller.getCinemaEditPage(id.toString(), model)
        assertThat(redirect.equals("cinemas/edit"))
        assertThat(model.getValue("action") == "Edit")
    }

    @Test
    @Throws(Exception::class)
    fun shouldReturnCinemasBrowsePage() {
        val cinema = Cinema()
        val id = UUID.randomUUID()
        cinema.id = id
        cinema.name = "test"
        controller.updateCinema(cinema)
        val model = ExtendedModelMap()
        val redirect = controller.getCinemasBrowsePage(model)
        assertThat(redirect.equals("cinemas/list"))
        assertThat(model.getValue("cinemas")).isNotNull
    }

    @Test
    @Sql("/cinemasFill.sql")
    fun mvcTest() {
        mvc.perform(get("/api/json/cinemas/all")
                .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk)
    }

//    @Test
//    @Throws(Exception::class)
//    fun shouldDeleteCinema() {
//        val cinema = Cinema()
//        val id = UUID.randomUUID()
//        cinema.id = id
//        cinema.name = "test"
//        controller.updateCinema(cinema)
//        var responce = HttpResponseFactory
//        apiController.deleteCinema(id.toString(), Http)
//        val cinemaFromBase = controller.cinemaService.findById(id)
//        assertThat(cinemaFromBase?.equals(cinema))
//    }
}
